@@ -7,7 +7,6 @@ from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from ecs_composex.common.settings import ComposeXSettings
-    from ecs_composex.mods_manager import ModManager
 
 from botocore.exceptions import ClientError
 from compose_x_common.aws import get_assume_role_session
@@ -40,7 +39,7 @@ from troposphere.ecs import (
 from troposphere.logs import LogGroup
 
 from ecs_composex.common import LOG, add_update_mapping
-from ecs_composex.compose.compose_services.helpers import (
+from ecs_composex.compose.compose_services.service_logging.helpers import (
     get_closest_valid_log_retention_period,
 )
 from ecs_composex.ecs import metadata
@@ -63,11 +62,10 @@ MANAGED_KMS_KEY_NAME = "ecs-cluster-logging-cmk"
 MANAGED_S3_BUCKET_NAME = "ecs-cluster-logging-bucket"
 
 
-def add_ecs_cluster(root_stack, settings):
+def add_ecs_cluster(settings):
     """
     Function to create the ECS Cluster.
 
-    :param ecs_composex.common.stacks.ComposeXStack root_stack:
     :param ecs_composex.common.settings.ComposeXSettings settings:
     """
     if keyisset("x-aws-cluster", settings.compose_content):
@@ -75,10 +73,12 @@ def add_ecs_cluster(root_stack, settings):
         LOG.info("x-aws-cluster was set. Overriding any defined x-cluster settings")
     if not keyisset(EcsCluster.res_key, settings.compose_content):
         LOG.info("No cluster information provided. Creating a new one")
-        cluster = EcsCluster(root_stack)
+        cluster = EcsCluster(settings.root_stack)
     elif isinstance(settings.compose_content[RES_KEY], dict):
-        cluster = EcsCluster(root_stack, settings.compose_content[EcsCluster.res_key])
-        cluster.set_from_definition(root_stack, settings.session, settings)
+        cluster = EcsCluster(
+            settings.root_stack, settings.compose_content[EcsCluster.res_key]
+        )
+        cluster.set_from_definition(settings.root_stack, settings.session, settings)
     else:
         raise LookupError("Unable to determine what to do for x-cluster")
     settings.ecs_cluster = cluster
@@ -354,7 +354,7 @@ class EcsCluster:
                             if keyisset("AllowKmsKeyReuse", self.parameters)
                             else Sub(
                                 f"arn:${{{AWS_PARTITION}}}:logs:${{{AWS_REGION}}}:${{{AWS_ACCOUNT_ID}}}:"
-                                f"log-group:/ecs/execute-logs/${{CLUSTER_NAME}}*",
+                                "log-group:/ecs/execute-logs/${CLUSTER_NAME}*",
                                 CLUSTER_NAME=cluster_name,
                             )
                         }
@@ -396,7 +396,7 @@ class EcsCluster:
             },
             "Settings": {
                 "Alias": Sub(
-                    f"alias/ecs/execute-logs/${{CLUSTER_NAME}}",
+                    "alias/ecs/execute-logs/${CLUSTER_NAME}",
                     CLUSTER_NAME=cluster_name,
                 )
             },
@@ -413,7 +413,7 @@ class EcsCluster:
         self.log_group = LogGroup(
             "EcsExecLogGroup",
             LogGroupName=Sub(
-                f"/ecs/execute-logs/${{CLUSTER_NAME}}",
+                "/ecs/execute-logs/${CLUSTER_NAME}",
                 CLUSTER_NAME=cluster_name,
             ),
             RetentionInDays=120
@@ -486,7 +486,7 @@ class EcsCluster:
             settings.compose_content["x-s3"][MANAGED_S3_BUCKET_NAME] = bucket_config
         log_configuration["S3BucketName"] = f"x-s3::{MANAGED_S3_BUCKET_NAME}"
         log_configuration["S3KeyPrefix"] = Sub(
-            f"ecs/execute-logs/${{CLUSTER_NAME}}/", CLUSTER_NAME=cluster_name
+            "ecs/execute-logs/${CLUSTER_NAME}/", CLUSTER_NAME=cluster_name
         )
         log_configuration["S3EncryptionEnabled"] = True
 
