@@ -1,10 +1,17 @@
-﻿#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
-# Copyright 2020-2021 John Mille <john@compose-x.io>
+# Copyright 2020-2022 John Mille <john@compose-x.io>
 
 """
 Module to handle the creation of the root EFS stack
 """
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from ecs_composex.common.settings import ComposeXSettings
+    from ecs_composex.mods_manager import XResourceModule
+
 import warnings
 
 from troposphere import GetAtt, Ref, Select, Sub
@@ -12,22 +19,14 @@ from troposphere.ec2 import SecurityGroup
 from troposphere.efs import FileSystem, MountTarget
 
 from ecs_composex.common import build_template
-from ecs_composex.common.compose_resources import (
-    XResource,
+from ecs_composex.common.stacks import ComposeXStack
+from ecs_composex.compose.x_resources.helpers import (
     set_lookup_resources,
     set_new_resources,
     set_resources,
-    set_use_resources,
-)
-from ecs_composex.common.stacks import ComposeXStack
-from ecs_composex.efs.efs_params import (
-    FS_ARN,
-    FS_ID,
-    FS_MNT_PT_SG_ID,
-    FS_PORT,
-    MOD_KEY,
-    RES_KEY,
 )
+from ecs_composex.compose.x_resources.network_x_resources import NetworkXResource
+from ecs_composex.efs.efs_params import FS_ARN, FS_ID, FS_MNT_PT_SG_ID, FS_PORT
 from ecs_composex.resources_import import import_record_properties
 from ecs_composex.vpc.vpc_params import STORAGE_SUBNETS, VPC_ID
 
@@ -51,28 +50,6 @@ def create_efs_stack(settings, new_resources):
             GroupDescription=Sub(f"SG for EFS {res.cfn_resource.title}"),
             VpcId=Ref(VPC_ID),
         )
-        if settings.vpc_imported:
-            for count, az in enumerate(
-                settings.subnets_mappings[STORAGE_SUBNETS.title]["Azs"]
-            ):
-                template.add_resource(
-                    MountTarget(
-                        f"{res.logical_name}MountPoint{az.title().strip().split('-')[-1]}",
-                        FileSystemId=Ref(res.cfn_resource),
-                        SecurityGroups=[GetAtt(res.db_sg, "GroupId")],
-                        SubnetId=Select(count, Ref(STORAGE_SUBNETS)),
-                    )
-                )
-        else:
-            for count, az in enumerate(settings.aws_azs):
-                template.add_resource(
-                    MountTarget(
-                        f"{res.logical_name}MountPoint{az['ZoneName'].title().strip().split('-')[-1]}",
-                        FileSystemId=Ref(res.cfn_resource),
-                        SecurityGroups=[GetAtt(res.db_sg, "GroupId")],
-                        SubnetId=Select(count, Ref(STORAGE_SUBNETS)),
-                    )
-                )
         template.add_resource(res.cfn_resource)
         template.add_resource(res.db_sg)
         res.init_outputs()
@@ -81,23 +58,27 @@ def create_efs_stack(settings, new_resources):
     return template
 
 
-class Efs(XResource):
+class Efs(NetworkXResource):
     """
     Class to represent a Filesystem
     """
 
     subnets_param = STORAGE_SUBNETS
 
-    def __init__(self, name, definition, module_name, settings, mapping_key=None):
+    def __init__(
+        self, name, definition, module: XResourceModule, settings: ComposeXSettings
+    ):
         self.db_sg = None
         self.db_secret = None
         self.mnt_targets = []
         self.access_points = []
         self.volume = definition["Volume"]
-        super().__init__(
-            name, definition, module_name, settings, mapping_key=mapping_key
-        )
+        super().__init__(name, definition, module, settings)
         self.set_override_subnets()
+        self.ref_parameter = FS_ID
+        self.arn_parameter = FS_ARN
+        self.security_group_param = FS_MNT_PT_SG_ID
+        self.port_param = FS_PORT
 
     def init_outputs(self):
         """
@@ -106,16 +87,16 @@ class Efs(XResource):
         self.output_properties = {
             FS_ID: (self.logical_name, self.cfn_resource, Ref, None),
             FS_ARN: (
-                self.logical_name,
+                f"{self.logical_name}{FS_ARN.return_value}",
                 self.cfn_resource,
                 GetAtt,
                 FS_ARN.return_value,
             ),
             FS_PORT: (
                 f"{self.logical_name}{FS_PORT.title}",
-                FS_PORT,
-                Ref,
                 None,
+                FS_PORT.Default,
+                False,
             ),
             FS_MNT_PT_SG_ID: (
                 f"{self.logical_name}{FS_MNT_PT_SG_ID.return_value}",
@@ -125,26 +106,57 @@ class Efs(XResource):
             ),
         }
 
+    def update_from_vpc(self, vpc_stack, settings=None):
+        """
+        Override for EFS to update settings from VPC Stack
+
+        :param ecs_composex.vpc.vpc_stack.XStack vpc_stack:
+        :param ecs_composex.common.settings.ComposeXSettings settings:
+        :return:
+        """
+        subnets_params = self.subnets_param
+        if self.subnets_override:
+            for subnet_az in vpc_stack.vpc_resource.azs:
+                if subnet_az.title == self.subnets_override:
+                    subnets_params = subnet_az
+                    break
+            else:
+                raise KeyError(
+                    f"{self.module.res_key}.{self.name} - "
+                    f"Override subnet name {self.subnets_override} is not defined in x-vpc",
+                    list(vpc_stack.vpc_resource.azs.keys()),
+                )
+        for count, az in enumerate(vpc_stack.vpc_resource.azs[subnets_params]):
+            self.stack.stack_template.add_resource(
+                MountTarget(
+                    f"{self.logical_name}MountPoint{az.title().strip().split('-')[-1]}",
+                    FileSystemId=Ref(self.cfn_resource),
+                    SecurityGroups=[GetAtt(self.db_sg, "GroupId")],
+                    SubnetId=Select(count, Ref(STORAGE_SUBNETS)),
+                )
+            )
+
 
 class XStack(ComposeXStack):
     """
     Class to represent the root for EFS
     """
 
-    def __init__(self, name, settings, **kwargs):
-        set_resources(settings, Efs, RES_KEY, MOD_KEY)
-        x_resources = settings.compose_content[RES_KEY].values()
-        new_resources = set_new_resources(x_resources, RES_KEY, False)
-        lookup_resources = set_lookup_resources(x_resources, RES_KEY)
-        use_resources = set_use_resources(x_resources, RES_KEY, False)
+    def __init__(
+        self, name, settings: ComposeXSettings, module: XResourceModule, **kwargs
+    ):
+        set_resources(settings, Efs, module)
+        x_resources = settings.compose_content[module.res_key].values()
+        new_resources = set_new_resources(x_resources, False)
+        lookup_resources = set_lookup_resources(x_resources)
         if new_resources:
             stack_template = create_efs_stack(settings, new_resources)
             super().__init__(name, stack_template, **kwargs)
         else:
             self.is_void = True
-        if lookup_resources or use_resources:
+        if lookup_resources:
             warnings.warn(
-                f"{RES_KEY} - Lookup not supported. You can only create new resources"
+                f"{module.res_key} - Lookup not supported. You can only create new resources at the moment"
             )
-        for resource in settings.compose_content[RES_KEY].values():
+        for resource in settings.compose_content[module.res_key].values():
             resource.stack = self
