@@ -1,43 +1,45 @@
-#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
-# Copyright 2020-2021 John Mille <john@compose-x.io>
+# Copyright 2020-2022 John Mille <john@compose-x.io>
 
 """
 Module to define the entry point for AWS Event Rules
 """
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+from ecs_composex.events.events_helpers import create_events_template
+
+if TYPE_CHECKING:
+    from ecs_composex.common.settings import ComposeXSettings
+    from ecs_composex.mods_manager import XResourceModule
+
 import warnings
 
 from ecs_composex.common import LOG, NONALPHANUM, build_template
-from ecs_composex.common.compose_resources import (
-    XResource,
+from ecs_composex.common.stacks import ComposeXStack
+from ecs_composex.compose.x_resources.helpers import (
     set_lookup_resources,
     set_new_resources,
     set_resources,
-    set_use_resources,
 )
-from ecs_composex.common.stacks import ComposeXStack
+from ecs_composex.compose.x_resources.services_resources import ServicesXResource
 from ecs_composex.ecs.ecs_params import CLUSTER_NAME, FARGATE_VERSION
-from ecs_composex.events.events_params import MOD_KEY, RES_KEY
-from ecs_composex.events.events_template import create_events_template
 
 
-def validate_service_definition(service):
-    required_keys = ["name", "TaskCount"]
-    if not set(required_keys).issubset(service):
-        raise KeyError(
-            "Services definition must contain at least",
-            required_keys,
-            "Got",
-            service.keys(),
-        )
-
-
-class Rule(XResource):
+class Rule(ServicesXResource):
     """
     Class to define an Event Rule
     """
 
     def handle_families_targets_expansion(self, service, settings):
+        """
+        Method to list all families and services that are targets of the resource.
+        Allows to implement family and service level association to resource
+
+        :param dict service: Service definition in compose file
+        :param ecs_composex.common.settings.ComposeXSettings settings: Execution settings
+        """
         the_service = [s for s in settings.services if s.name == service["name"]][0]
         for family_name in the_service.families:
             family_name = NONALPHANUM.sub("", family_name)
@@ -52,7 +54,7 @@ class Rule(XResource):
                     )
                 )
 
-    def set_services_targets(self, settings):
+    def set_services_targets_from_list(self, settings):
         """
         Override method to map services and families targets of the services defined specifically for
         events
@@ -66,7 +68,6 @@ class Rule(XResource):
             LOG.info(f"No services defined for {self.name}")
             return
         for service in self.services:
-            validate_service_definition(service)
             service_name = service["name"]
             if service_name in settings.families and service_name not in [
                 f[0].name for f in self.families_targets
@@ -89,13 +90,71 @@ class Rule(XResource):
             elif service_name in [s.name for s in settings.services]:
                 self.handle_families_targets_expansion(service, settings)
 
+    def handle_families_targets_expansion_dict(
+        self, service_name, service, settings
+    ) -> None:
+        """
+        Method to list all families and services that are targets of the resource.
+        Allows to implement family and service level association to resource
+
+        :param str service_name:
+        :param dict service: Service definition in compose file
+        :param ecs_composex.common.settings.ComposeXSettings settings: Execution settings
+        """
+        the_service = [s for s in settings.services if s.name == service_name][0]
+        for family_name in the_service.families:
+            family_name = NONALPHANUM.sub("", family_name)
+            if family_name not in [f[0].name for f in self.families_targets]:
+                self.families_targets.append(
+                    (
+                        settings.families[family_name],
+                        False,
+                        [the_service],
+                        service["TaskCount"],
+                        service,
+                    )
+                )
+
+    def set_services_targets_from_dict(self, settings):
+        """
+        Deals with services set as a dict
+
+        :param settings:
+        :return:
+        """
+        for service_name, service_def in self.services.items():
+            if service_name in settings.families and service_name not in [
+                f[0].name for f in self.families_targets
+            ]:
+                self.families_targets.append(
+                    (
+                        settings.families[service_name],
+                        True,
+                        settings.families[service_name].services,
+                        service_def["TaskCount"],
+                        service_def,
+                    )
+                )
+            elif service_name in settings.families and service_name in [
+                f[0].name for f in self.families_targets
+            ]:
+                LOG.debug(
+                    f"{self.module.res_key}.{self.name} - Family {service_name} has already been added. Skipping"
+                )
+            elif service_name in [s.name for s in settings.services]:
+                self.handle_families_targets_expansion_dict(
+                    service_name, service_def, settings
+                )
+
 
 class XStack(ComposeXStack):
     """
     Class to handle events stack
     """
 
-    def __init__(self, title, settings, **kwargs):
+    def __init__(
+        self, title, settings: ComposeXSettings, module: XResourceModule, **kwargs
+    ):
         """
         Method to initialize the XStack for Events
 
@@ -103,12 +162,16 @@ class XStack(ComposeXStack):
         :param ecs_composex.common.settings.ComposeXSettings settings: Execution settings
         :param dict kwargs:
         """
-        set_resources(settings, Rule, RES_KEY, MOD_KEY)
-        x_resources = settings.compose_content[RES_KEY].values()
-        new_resources = set_new_resources(x_resources, RES_KEY, False)
-        lookup_resources = set_lookup_resources(x_resources, RES_KEY)
-        use_resources = set_use_resources(x_resources, RES_KEY, False)
-        if new_resources or use_resources:
+        set_resources(settings, Rule, module)
+        x_resources = settings.compose_content[module.res_key].values()
+        lookup_resources = set_lookup_resources(x_resources)
+        if lookup_resources:
+            warnings.warn(
+                f"{module.res_key} does not support Lookup/Use. You can only create new resources"
+            )
+
+        new_resources = set_new_resources(x_resources, False)
+        if new_resources:
             stack_template = build_template(
                 "Events rules for ComposeX",
                 [CLUSTER_NAME, FARGATE_VERSION],
@@ -117,7 +180,3 @@ class XStack(ComposeXStack):
             create_events_template(self, settings, new_resources)
         else:
             self.is_void = True
-        if lookup_resources or use_resources:
-            warnings.warn(
-                f"{RES_KEY} does not support Lookup. You can only create new resources"
-            )
